LIBRARY ieee;
USE ieee.std_logic_1164.ALL;
USE ieee.numeric_std.ALL;

ENTITY ram_ctrl_test IS
	PORT (
		clk, reset : IN std_logic;
		sw : IN std_logic_vector(3 DOWNTO 0);
		btn : IN std_logic_vector(2 DOWNTO 0);
		led : OUT std_logic_vector(3 DOWNTO 0);
		ad : OUT std_logic_vector(18 DOWNTO 0);
		we_n, oe_n : OUT std_logic;
		dio : INOUT std_logic_vector(7 DOWNTO 0);
		ce_n : OUT std_logic
	);
END ram_ctrl_test;

ARCHITECTURE arch OF ram_ctrl_test IS
	CONSTANT ADDR_W : INTEGER := 19;
	CONSTANT DATA_W : INTEGER := 8;
	SIGNAL addr : std_logic_vector(ADDR_W - 1 DOWNTO 0);
	SIGNAL data_f2s, data_s2f : 
	std_logic_vector(DATA_W - 1 DOWNTO 0);
	SIGNAL mem, rw : std_logic;
	SIGNAL data_reg : std_logic_vector(3 DOWNTO 0);
	SIGNAL db_btn : std_logic_vector(2 DOWNTO 0);
BEGIN
	ctrl_unit : ENTITY work.sram_ctrl
		PORT MAP(
			clk => clk, reset => reset, mem => mem, rw => rw, 
			addr => addr, data_f2s => data_f2s, 
			ready => OPEN, data_s2f_r => data_s2f, 
			data_s2f_ur => OPEN, ad => ad, we_n => we_n, 
			oe_n => oe_n, dio => dio, ce_n => ce_n
		);
			debounce_unit0 : ENTITY work.debounce
				PORT MAP(
					clk => clk, reset_n => reset, button => btn(0), 
					result => db_btn(0)
				);
					debounce_unit1 : ENTITY work.debounce
						PORT MAP(
							clk => clk, reset_n => reset, button => btn(1), 
							result => db_btn(1)
						);
							debounce_unit2 : ENTITY work.debounce
								PORT MAP(
									clk => clk, reset_n => reset, button => btn(2), 
									result => db_btn(2)
								);
									--data registers
									PROCESS (clk)
	BEGIN
		IF (clk'event AND clk = '1') THEN
			IF (db_btn(0) = '1') THEN
				data_reg <= sw;
			END IF;
		END IF;
	END PROCESS;
	-- address
	addr <= "000000000000000" & sw;
	-- command
	PROCESS (db_btn, data_reg)
		BEGIN
			data_f2s <= (OTHERS => '0');
			IF (db_btn(1) = '1') THEN -- write
				mem <= '1';
				rw <= '0';
				data_f2s <= "0000" & data_reg;
			ELSIF (db_btn(2) = '1') THEN -- read
				mem <= '1';
				rw <= '1';
			ELSE
				mem <= '0';
				rw <= '1';
			END IF;
		END PROCESS;
		led <= data_s2f(3 DOWNTO 0);
END arch;